@@ -71,7 +71,7 @@ use constant
 %EXPORT_TAGS = (constants => ['SOCKS_WANT_READ', 'SOCKS_WANT_WRITE', @EXPORT_OK]);
 $SOCKS_ERROR = new IO::Socket::Socks::Error;
 
-$VERSION = '0.62_1';
+$VERSION = '0.62_2';
 $SOCKS5_RESOLVE = 1;
 $SOCKS4_RESOLVE = 0;
 $SOCKS_DEBUG = $ENV{SOCKS_DEBUG};
@@ -949,6 +949,7 @@ sub accept
         ${*$client}->{SOCKS}->{Version}     = ${*$self}->{SOCKS}->{Version};
         ${*$client}->{SOCKS}->{AuthMethods} = ${*$self}->{SOCKS}->{AuthMethods};
         ${*$client}->{SOCKS}->{UserAuth}    = ${*$self}->{SOCKS}->{UserAuth};
+        ${*$client}->{SOCKS}->{Resolve}     = ${*$self}->{SOCKS}->{Resolve};
         ${*$client}->{SOCKS}->{ready} = 0;
         $client->blocking($self->blocking); # temporarily
         
@@ -1278,7 +1279,7 @@ sub _socks5_accept_command
         $debug->show('Server Recv: ');
     }
     
-    @{${*$self}->{SOCKS}->{COMMAND}} = ($cmd, $dstaddr, $dstport);
+    @{${*$self}->{SOCKS}->{COMMAND}} = ($cmd, $dstaddr, $dstport, $atyp);
 
     return 1;
 }
@@ -1405,9 +1406,12 @@ sub _socks4_accept_command
         );
     }
     
+    my $atyp = ADDR_IPV4;
+    
     if($resolve && $dstaddr =~ /^0\.0\.0\.[1-9]/)
     { # socks4a
-        my $dsthost = '';
+        $dstaddr = '';
+        $atyp = ADDR_DOMAINNAME;
         
         while(1)
         {
@@ -1416,7 +1420,7 @@ sub _socks4_accept_command
             
             if($c ne "\0")
             {
-                $dsthost .= $c;
+                $dstaddr .= $c;
             }
             else
             {
@@ -1427,12 +1431,10 @@ sub _socks4_accept_command
         if($debug && !$self->_debugged(++$debugs))
         {
             $debug->add(
-                dsthost => $dsthost,
+                dsthost => $dstaddr,
                 null => 0
             );
         }
-        
-        $dstaddr = join('.', unpack('C4', (gethostbyname($dsthost))[4]));
     }
     
     if($debug && !$self->_debugged(++$debugs))
@@ -1460,7 +1462,7 @@ sub _socks4_accept_command
         return;
     }
     
-    @{${*$self}->{SOCKS}->{COMMAND}} = ($cmd, $dstaddr, $dstport);
+    @{${*$self}->{SOCKS}->{COMMAND}} = ($cmd, $dstaddr, $dstport, $atyp);
     
     return 1;
 }
@@ -2452,9 +2454,9 @@ After you call accept() the client has sent the command they want
 you to process.  This function should be called on the socket returned
 by accept(). It returns a reference to an array with the following format:
 
-  [ COMMAND, HOST, PORT ]
+  [ COMMAND, ADDRESS, PORT, ADDRESS TYPE ]
 
-=head3 command_reply( REPLY CODE, HOST, PORT )
+=head3 command_reply( REPLY CODE, ADDRESS, PORT )
 
 After you call command() the client needs to be told what the result
 is.  The REPLY CODE is one of the constants as follows (integer value):
